pub use pomodoro::Pomodoro;

pub mod pomodoro {
    use directories::ProjectDirs;
    use rodio::{Decoder, Device, Source};
    use std::fs::{self, File};
    use std::io::BufReader;
    use std::path::{Path, PathBuf};

    #[derive(Copy, Clone)]
    pub struct Alert<'a> {
        path: &'a Path,
        device: &'a Device,
    }

    impl Alert<'_> {
        pub fn new<'a, P: AsRef<Path>>(path: &'a P, device: &'a Device) -> Alert<'a> {
            Alert {
                path: path.as_ref(),
                device,
            }
        }

        pub fn load<'a, 'b, P: AsRef<Path>>(&'a mut self, _path: &'b P) {
            unimplemented!()
        }

        pub fn play(&self) {
            let file = File::open(self.path).unwrap();
            let source = Decoder::new(BufReader::new(file)).unwrap();

            rodio::play_raw(self.device, source.convert_samples());
        }
    }

    #[derive(PartialEq, Eq)]
    pub enum State {
        Work,
        ShortBreak,
        LongBreak,
        Inactive,
    }

    pub struct Config {
        pub work_time: u64,
        pub short_break: u64,
        pub long_break: u64,
        pub data_directory: PathBuf,
    }

    impl Default for Config {
        fn default() -> Self {
            let dirs = ProjectDirs::from("moe", "paoda", "domasi").unwrap();

            let data_directory = dirs.data_dir().to_owned();

            if !data_directory.exists() {
                fs::create_dir_all(&data_directory).unwrap();
            }

            Config {
                work_time: 1,
                short_break: 1,
                long_break: 1,
                data_directory,
            }
        }
    }

    pub struct Pomodoro<'a> {
        count: u64,
        state: State,
        alert: Alert<'a>,
    }

    impl Pomodoro<'_> {
        pub fn new<'a>(alert: &'a Alert) -> Pomodoro<'a> {
            Pomodoro {
                count: 0,
                state: State::Inactive,
                alert: *alert,
            }
        }

        fn next(&mut self) {
            match self.state {
                State::Work => {
                    self.count += 1;

                    if (self.count % 4) == 0 {
                        self.state = State::LongBreak;
                    } else {
                        self.state = State::ShortBreak;
                    }
                }
                State::LongBreak => self.state = State::Inactive,
                State::ShortBreak => self.state = State::Work,
                State::Inactive => self.state = State::Work,
            }
        }

        async fn wait(minutes: u64) {
            use std::time::Duration;

            let duration = Duration::from_secs(minutes * 5);
            async_std::task::sleep(duration).await;
        }

        pub async fn start(&mut self, config: Config) {
            loop {
                self.next();

                self.alert.play();

                match self.state {
                    State::Work => {
                        println!("Start Work.");
                        Self::wait(config.work_time).await;
                    }
                    State::ShortBreak => {
                        println!("Start Short Break.");
                        Self::wait(config.short_break).await;
                    }
                    State::LongBreak => {
                        println!("Start Long Break.");
                        Self::wait(config.long_break).await;
                    }
                    State::Inactive => {
                        println!("Pomodoro Cycle is complete");
                        break;
                    }
                }
            }
        }

        pub fn completed(&self) -> u64 {
            self.count / 4
        }
    }
}
